/*
 * (C)opyright 2006 Nico Golde <nico@ngolde.de>
 * (C)opyright 2006 Andreas Krennmair <ak@synflood.at>
 * See LICENSE file for license details
 */

#include "list.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

wordlist_t * new_wordlist(void) {
	wordlist_t * l = malloc(sizeof(wordlist_t));
	if (!l) {
		/* allocation error */
		return NULL;
	}
	l->top = l->last = NULL;
	l->count = 0;
	return l;
}

void delete_word_r(word_t * e) {
	/* TODO: transform this to an iteration */
	if (e) {
		delete_word_r(e->next);
		if (e->word)
			free(e->word);
		free(e);
	}
}

void delete_list(wordlist_t * l) {
	delete_word_r(l->top);
	free(l);
}

void append_word(wordlist_t * list, char * word) {
	word_t * e;
	if (!list) return;

	e = malloc(sizeof(word_t));
	if (!e) return; /* allocation error */

	e->word = strdup(word);
	if (!e->word) return; /* allocation error */

	e->next = NULL;
	if (list->top) {
		list->last->next = e;
		list->last = list->last->next;
	} else {
		list->top = list->last = e;
	}
	++list->count;
}

void fill_wordlist_from_file(wordlist_t * list, char * filename) {
	if (list) {
		FILE * f = fopen(filename,"r");
		if (!f) {
			fprintf(stderr,"error: couldn't open `%s': %s\n",filename,strerror(errno));
			exit(EXIT_FAILURE);
			return;
		}
		while (!feof(f)) {
			char word[1024];
			fscanf(f,"%1023s",word);
			if (!feof(f)) {
				append_word(list,word);
			}
		}
		fclose(f);
	}
}
