/*
 * (C)opyright 2006 Nico Golde <nico@ngolde.de>
 * (C)opyright 2006 Andreas Krennmair <ak@synflood.at>
 * See LICENSE file for license details
 */

#include "url.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

url_t * parse_url(char *name) {
	url_t * url;
	char * p, * p1, * p2;
	int found_port = 0;

	if (!name) return NULL;

	url = malloc(sizeof(url_t));
	if (!url) return NULL;

	url->protocol = PROTO_INVALID;
	url->host = NULL;
	url->port = 0;
	url->resource = NULL;

	if (strncmp(name,"http://",7)==0) {
		url->protocol = PROTO_HTTP;
		url->port = 80;
		p = name + 7;
		/* fprintf(stderr,"protocol is HTTP\n"); */
	} else if (strncmp(name,"https://",8)==0) {
		url->protocol = PROTO_HTTPS;
		url->port = 443;
		p = name + 8;
		/* fprintf(stderr,"protocol is HTTPS\n"); */
	} else {
		free_url(url);
		/* fprintf(stderr,"invalid protocol: %s\n",name); */
		return NULL;
	}

	p1 = p;
start_switch:
	while (*p1 != ':' && *p1 != '/' && *p1 != '\0') {
		/* fprintf(stderr,"got character: %c\n",*p1); */
		p1++;
	}

	switch (*p1) {
		case ':': /* found port */
			if (!found_port) {
				url->host = malloc(p1-p+1);
				if (!url->host) {
					/* fprintf(stderr,"url->host malloc failed\n"); */
					free_url(url);
					return NULL;
				}
				memcpy(url->host,p,p1-p);
				url->host[p1-p] = '\0';
				found_port = 1;
				p1++;
				p2 = p1;
				while (*p2 != '/' && *p2 != '\0') { 
					/* fprintf(stderr,"-got character: %c\n",*p2); */
					p2++;
				}
				{
					char * portstr = alloca(p2-p1+1);
					memcpy(portstr,p1,p2-p1);
					portstr[p2-p1] = '\0';
					/* fprintf(stderr,"port = %s (%u)\n",portstr,atoi(portstr)); */
					url->port = strtol(portstr, NULL, 10);
				}
				goto start_switch; /* continue parsing. A further port will be ignored */
				break;
			}
		case '/': /* found resource */
			if (!found_port) {
				url->host = malloc(p1-p+1);
				if (!url->host) {
					/* fprintf(stderr,"url->host malloc failed 2\n"); */
					free_url(url);
					return NULL;
				}
				memcpy(url->host,p,p1-p);
				url->host[p1-p] = '\0';
			}
			url->resource = malloc(strlen(p1)+1);
			if (!url->resource) {
				/* fprintf(stderr,"url->resource malloc failed\n"); */
				free_url(url);
				return NULL;
			}
			strcpy(url->resource,p1);
			break;
		case '\0': /* found end of string */
			if (!found_port) {
				url->host = malloc(p1-p+1);
				if (!url->host) {
					/* fprintf(stderr,"url->host malloc failed 3\n"); */
					free_url(url);
					return NULL;
				}
				memcpy(url->host,p,p1-p);
				url->host[p1-p] = '\0';
			}
			url->resource = strdup("/");
			if (!url->resource) {
				free_url(url);
				return NULL;
			}
			break;
	}

	return url;
}

void free_url(url_t * url) {
	if (url) {
		if (url->host)
			free(url->host);
		if (url->resource)
			free(url->resource);
		free(url);
	}
}

void url_to_str(char * str, size_t n, url_t * url) {
	if (!url || !str) return;
	if ((url->protocol == PROTO_HTTP && url->port == 80) || (url->protocol == PROTO_HTTPS && url->port == 443)) {
		snprintf(str,n,"%s://%s%s",url->protocol == PROTO_HTTP ? "http" : "https", url->host, url->resource);
	} else {
		snprintf(str,n,"%s://%s:%u%s",url->protocol == PROTO_HTTP ? "http" : "https", url->host, url->port, url->resource);
	}
}

#ifdef _TEST_URL

#include <assert.h>

int main(void) {
	url_t * u = parse_url("http://slashdot.org/");
	assert(u != NULL);
	assert(u->protocol == PROTO_HTTP);
	assert(u->port == 80);
	assert(strcmp(u->host,"slashdot.org")==0);
	assert(strcmp(u->resource,"/")==0);
	free_url(u);

	u = parse_url("https://foobar:8081");
	assert(u != NULL);
	assert(u->protocol == PROTO_HTTPS);
	assert(u->port == 8081);
	assert(strcmp(u->host,"foobar")==0);
	assert(strcmp(u->resource,"/")==0);
	free_url(u);

	u = parse_url("http://blafasel:8000/resource/asdf");
	assert(u != NULL);
	assert(u->protocol == PROTO_HTTP);
	assert(u->port == 8000);
	assert(strcmp(u->host,"blafasel")==0);
	assert(strcmp(u->resource,"/resource/asdf")==0);
	free_url(u);

	u = parse_url("QWERT");
	assert(u == NULL);

	u = parse_url("http://");
	assert(u != NULL);
	assert(u->protocol == PROTO_HTTP);
	assert(u->port == 80);
	assert(strcmp(u->host,"")==0);
	assert(strcmp(u->resource,"/")==0);
	free_url(u);

	return EXIT_SUCCESS;
}

#endif
